/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.services.bootstrap;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;

import org.ow2.jonas.configuration.DeploymentPlanDeployer;
import org.ow2.jonas.lib.bootstrap.JProp;
import org.ow2.jonas.properties.ServerProperties;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.archive.impl.ArchiveManager;
import org.ow2.util.ee.deploy.api.deployable.IDeployable;
import org.ow2.util.ee.deploy.api.deployer.IDeployerManager;
import org.ow2.util.ee.deploy.impl.helper.DeployableHelper;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Allows to deploy or undeploy all bundles for a particular deployment plan. At the
 * server initialization, deploy all JOnAS services defined in the configuration.
 * @author Francois Fornaciari
 */
public class JOnASDeploymentPlanDeployer implements DeploymentPlanDeployer {

    /**
     * DeployerManager reference.
     */
    private IDeployerManager deployerManager = null;

    /**
     * ServerProperties service object.
     */
    private ServerProperties serverProperties = null;

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(JOnASDeploymentPlanDeployer.class);

    /**
     * Property name for the service name.
     */
    private static final String JONAS_SERVICE = "jonas.service";

    /**
     * JOnAS configuration directory.
     */
    private static final String DEPLOYMENT_CONFIG_DIR = JProp.getConfDir() + File.separator + "deployment";

    /**
     * {@inheritDoc}
     */
    public void deploy(final String name) throws Exception {
        String service = name;
        File configDir = new File(DEPLOYMENT_CONFIG_DIR);

        final String baseName = name + "-base";
        File[] base = configDir.listFiles(new FilenameFilter() {
            public boolean accept(final File dir, final String name) {
                return (name.equalsIgnoreCase(baseName.concat(".xml")));
            }
        });
        if (base.length == 1) {
            deploy(baseName);

            // Get the name of the user-defined implementation service
            String serviceClass = serverProperties.getValue(JONAS_SERVICE + "." + service + ".class");
            // The value follows this construction:
            // org.ow2.jonas.<service>.<implementation>.<...>
            // The implementation name is the 5th element.
            // TODO: method not easily extensible if the package name doesn't
            // start with org.ow2.jonas
            String implementationName = serviceClass.split("\\.")[4];

            // Deploy the <service>-<implementationName>.xml file
            service += "-" + implementationName;
        }

        File file = new File(DEPLOYMENT_CONFIG_DIR, service + ".xml");
        if (file.exists()) {
            deployDeploymentPlan(file);
        } else {
            throw new FileNotFoundException("Cannot deploy the deployment plan '" + name + "', file '" + file + "' not found");
        }
    }

    /**
     * {@inheritDoc}
     */
    public void undeploy(final String name) throws Exception {
        File file = new File(DEPLOYMENT_CONFIG_DIR, name + ".xml");
        if (file.exists()) {
            undeployDeploymentPlan(file);
        } else {
            throw new FileNotFoundException("Cannot undeploy the deployment plan '" + name + "', file '" + file + "' not found");
        }
    }

    /**
     * Deploy a deployment plan using its associated deployment plan file.
     * @param file File corresponding to the plan to deploy
     * @throws Exception If the deployment of the deployment plan fails
     */
    private void deployDeploymentPlan(final File file) throws Exception {
        logger.debug("Deploying the deployment plan for ''{0}''", file);

        // Get the deployable
        IArchive archive = ArchiveManager.getInstance().getArchive(file);
        if (archive == null) {
            throw new Exception("Ignoring invalid file '" + file + "'");
        }

        IDeployable<?> deployable = null;
        try {
            deployable = DeployableHelper.getDeployable(archive);
            deployerManager.deploy(deployable);
        } catch (Exception e) {
            throw new Exception("Cannot deploy the deployment plan for '" + file + "'", e);
        }
    }

    /**
     * Undeploy a deployment plan using its associated deployment plan file.
     * @param file File corresponding to the plan to undeploy
     * @throws Exception If the undeployment of the deployment plan fails
     */
    private void undeployDeploymentPlan(final File file) throws Exception {
        logger.debug("Undeploying the deployment plan for ''{0}''", file);

        // Get the deployable
        IArchive archive = ArchiveManager.getInstance().getArchive(file);
        if (archive == null) {
            throw new Exception("Ignoring invalid file '" + file + "'");
        }

        IDeployable<?> deployable = null;
        try {
            deployable = DeployableHelper.getDeployable(archive);
            deployerManager.undeploy(deployable);
        } catch (Exception e) {
            throw new Exception("Cannot undeploy the deployment plan for '" + file + "'", e);
        }
    }

    /**
     * @param deployerManager the deployerManager to set
     */
    public void setDeployerManager(final IDeployerManager deployerManager) {
        this.deployerManager = deployerManager;
    }

    /**
     * Set the server properties service.
     * @param serverProperties the serverProperties to set
     */
    public void setServerProperties(final ServerProperties serverProperties) {
        this.serverProperties = serverProperties;
    }
}
