/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.ws.cxf.easybeans.ext;

import org.apache.cxf.Bus;
import org.apache.cxf.jaxws.EndpointImpl;
import org.apache.cxf.jaxws.JaxWsServerFactoryBean;
import org.apache.cxf.jaxws.support.JaxWsImplementorInfo;
import org.apache.cxf.jaxws.support.JaxWsServiceFactoryBean;
import org.apache.cxf.resource.ClassLoaderResolver;
import org.apache.cxf.resource.ResourceManager;
import org.apache.cxf.wsdl.WSDLManager;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Level;
import org.ow2.easybeans.api.EZBContainerCallbackInfo;
import org.ow2.easybeans.api.Factory;
import org.ow2.easybeans.api.bean.info.IBeanInfo;
import org.ow2.easybeans.api.bean.info.IWebServiceInfo;
import org.ow2.easybeans.container.EmptyLifeCycleCallBack;
import org.ow2.easybeans.container.session.stateless.StatelessSessionFactory;
import static org.ow2.jonas.lib.util.Log.setComponentLogLevel;
import org.ow2.jonas.ws.cxf.BusCreator;
import org.ow2.jonas.ws.cxf.JOnASJaxWsImplementorInfo;
import org.ow2.jonas.ws.cxf.easybeans.EasyBeansEndpoint;
import org.ow2.jonas.ws.cxf.easybeans.EasyBeansJaxWsServerFactoryBean;
import org.ow2.jonas.ws.cxf.http.JOnASDestination;
import org.ow2.jonas.ws.cxf.jaxws.CXFWSEndpoint;
import org.ow2.jonas.ws.jaxws.IWebServiceDeploymentManager;
import org.ow2.jonas.ws.jaxws.IWebServiceEndpoint;
import org.ow2.jonas.ws.jaxws.PortMetaData;
import org.ow2.jonas.ws.jaxws.WSException;
import org.ow2.jonas.ws.jaxws.util.JAXWSClassUtils;
import org.ow2.jonas.ws.publish.PublicationInfo;
import org.ow2.jonas.ws.publish.WSDLPublisherException;
import org.ow2.jonas.ws.publish.WSDLPublisherManager;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.WSDLException;
import javax.wsdl.extensions.soap12.SOAP12Address;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.xml.namespace.QName;
import javax.xml.ws.Endpoint;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Callback for CXF.
 * @author Guillaume Sauthier
 * @author Florent Benoit
 */
public class LifeCycleCallback extends EmptyLifeCycleCallBack {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(LifeCycleCallback.class);

    /**
     * Easybeans internal Bus.
     */
    private Bus bus = null;

    /**
     * The WS deployment manager.
     */
    private IWebServiceDeploymentManager manager;

    /**
     * List of deployed endpoints.
     */
    private List<IWebServiceEndpoint> endpoints = new ArrayList<IWebServiceEndpoint>();

    /**
     * WSDL Publication tool.
     */
    private WSDLPublisherManager publisherManager;

    /**
     * @param manager the manager to set
     */
    public void setManager(final IWebServiceDeploymentManager manager) {
        this.manager = manager;
    }


    /**
     * Called when container is starting.
     * @param info some information on the container which is starting.
     */
    @Override
    public void start(final EZBContainerCallbackInfo info) {

        // Generic part ----------

        // Iterates over the factories
        Map<String, Factory<?, ?>> factories = info.getFactories();
        for (Factory<?, ?> f : factories.values()) {

            // Only for @Stateless
            if (f instanceof StatelessSessionFactory) {

                StatelessSessionFactory factory = (StatelessSessionFactory) f;

                Class<?> klass = factory.getBeanClass();

                // An EJB exposed as a JAX-WS web services HAS TO be annotated
                // with @WebService or @WebServiceProvider
                if (JAXWSClassUtils.isWebService(klass)) {

                    // Init CXF Bus if needed
                    if (bus == null) {
                        BusCreator creator = new BusCreator();
                        bus = creator.initBus();
                    }

                    // Extract merged webservices deploy time informations
                    IBeanInfo beanInfo = factory.getBeanInfo();
                    IWebServiceInfo webServiceInfo = beanInfo.getWebServiceInfo();

                    // Creates PortMetaData to hold deployment/runtime information
                    // about the web service
                    PortMetaData pmd = new PortMetaData();
                    String contextRoot = extractContextRoot(info.getArchive().getName());
                    pmd.setContextRoot(contextRoot);

                    // End of Generic Part -----------

                    EasyBeansEndpoint easybeansEndpoint = createEasyBeansEndpoint(factory, klass, webServiceInfo);
                    JaxWsImplementorInfo jaxWsInfo = easybeansEndpoint.getServiceFactory().getJaxWsImplementorInfo();


                    // find the URL pattern to use
                    String pattern = null;

                    if ((webServiceInfo != null) && (webServiceInfo.getEndpointAddress() != null)) {

                        // User defined address
                        pattern = webServiceInfo.getEndpointAddress();
                        if (!pattern.startsWith("/")) {
                            // Pattern needs to start with a '/'
                            pattern = "/" + pattern;
                        }
                    } else {

                        // Default naming strategy
                        String serviceName = jaxWsInfo.getServiceName().getLocalPart();
                        if (!"".equals(serviceName)) {
                            // default pattern : /{service-name}
                            pattern = "/" + serviceName;
                        } else {
                            // else /{class-name}Service
                            pattern = "/" + klass.getSimpleName() + "Service";
                        }
                    }
                    pmd.setUrlPattern(pattern);

                    logger.info("{0}", pmd);

                    // Wraps the EasyBeansEndpoint into our own IWSEndpoint implementation
                    CXFWSEndpoint endpoint = new CXFWSEndpoint(easybeansEndpoint,
                                                               IWebServiceEndpoint.EndpointType.EJB,
                                                               pmd);

                    // OK, We have a endpoint, in some way, we have to register it to the web container
                    // So here we should have an IWSDeploymentManager instance that we can use

                    try {
                        manager.registerWSEndpoint(endpoint);
                    } catch (WSException e) {
                        // Log an error
                        logger.error("Unable to register endpoint {0}", endpoint, e);

                        // Try to unregister previous stuff
                        try {
                            manager.unregisterWSEndpoint(endpoint);
                        } catch (WSException e1) {
                            logger.error("Unable to unregister endpoint {0} after a failed deployment", endpoint, e);
                        }
                        throw new RuntimeException("Endpoint not deployed completely", e);
                    }

                    endpoint.start();

                    // Attempt to publish this service's WSDL
                    publishWSDL(endpoint, jaxWsInfo, webServiceInfo);

                    // Add the endpoint in the list
                    endpoints.add(endpoint);

                    logger.info("Endpoint {0} inited", klass.getName());

                }
            }
        }
    }

    /**
     * Publish this service's WSDL using the WSDL publisher manager.
     * @param endpoint web service to be published
     * @param info Runtime info about the webservice being deployed
     * @param webServiceInfo XML infos
     */
    private void publishWSDL(CXFWSEndpoint endpoint,
                             JaxWsImplementorInfo info,
                             IWebServiceInfo webServiceInfo) {

        // Prepare the Publication resources
        PublicationInfo publicationInfo = new PublicationInfo();

        publicationInfo.setOriginalWsdlFilename(info.getWsdlLocation());

        if ((webServiceInfo != null) && (webServiceInfo.getWsdlPublicationDirectory() != null)) {
            File directory = new File(webServiceInfo.getWsdlPublicationDirectory());
            publicationInfo.setPublicationDirectory(directory);
        }

        // Get the Bus ...
        JOnASDestination destination = endpoint.getDestination();
        Bus bus = destination.getBus();
        WSDLManager wsdlManager = bus.getExtension(WSDLManager.class);

        try {
            Definition definition = wsdlManager.getDefinition(info.getWsdlLocation());
            updateDefinition(definition, info, endpoint.getPortMetaData());
            publisherManager.publish(definition, publicationInfo);
        } catch (WSDLPublisherException e) {
            // Log an error
            logger.error("Unable to publish WSDL of endpoint {0}", endpoint, e);
        } catch (WSDLException e) {
            logger.error("Unable to load WSDL of endpoint {0}", endpoint, e);
        }
    }

    /**
     * Update the given WSDL Definition.
     * @param definition WSDL document
     * @param info contains wsdl:service and wsdl:port names
     * @param portMetaData contains the update endpoint URL
     */
    private void updateDefinition(Definition definition,
                                  JaxWsImplementorInfo info,
                                  PortMetaData portMetaData) {

        QName serviceName = info.getServiceName();
        QName portName = info.getEndpointName();

        Service service = definition.getService(serviceName);
        Port port = service.getPort(portName.getLocalPart());

        if (portMetaData.getEndpointURL() != null) {
            updateSoapAddressLocation(port, portMetaData.getEndpointURL());
        }

        // TODO remove other services/port not implemented by this endpoint

    }

    /**
     * Update the soap:address extensibility element contained in the given port.
     * @param port wsdl:port instance
     * @param url new value
     */
    private void updateSoapAddressLocation(Port port, String url) {
        List extensions = port.getExtensibilityElements();
        for (Object extension : extensions) {
            if (extension instanceof SOAP12Address) {
                ((SOAP12Address)extension).setLocationURI(url);
            } else if (extension instanceof SOAPAddress) {
                ((SOAPAddress)extension).setLocationURI(url);
            }
        }
    }



    /**
     * Extract a context-root name from the archive filename.
     * @param filename archive filename
     * @return a context-root which is the last part of the filename minus the extension
     */
    private static String extractContextRoot(final String filename) {
        String context = filename.substring(filename.lastIndexOf(File.separator) + 1);
        context = context.substring(0, context.lastIndexOf('.'));
        return context;
    }

    /**
     * Configure CXF internals for the new JAXWS {@link Endpoint}.
     * @param factory The EasyBeans EJB factory (used to get the instances)
     * @param klass Bean's class
     * @param info Info about the port being deployed
     * @return a configured CXF {@link EndpointImpl} subclass.
     */
    private EasyBeansEndpoint createEasyBeansEndpoint(final StatelessSessionFactory factory,
                                                      final Class<?> klass,
                                                      final IWebServiceInfo info) {

        JaxWsImplementorInfo jaxWsImplementorInfo;
        if (info != null) {
            jaxWsImplementorInfo = new JOnASJaxWsImplementorInfo(klass, info);
        } else {
            jaxWsImplementorInfo = new JaxWsImplementorInfo(klass);
        }
        JaxWsServiceFactoryBean jaxWsServiceFactoryBean = new JaxWsServiceFactoryBean(jaxWsImplementorInfo);
        jaxWsServiceFactoryBean.setBus(this.bus);

        // The Bus can resolve file against the module Classloader
        ResourceManager manager = this.bus.getExtension(ResourceManager.class);
        manager.addResourceResolver(new ClassLoaderResolver(klass.getClassLoader()));

        // We need to call create() here in order to initialize the ServiceFactory
        // otherwise, CXF try to guess some values without using our JaxWsImplementorInfo object.
        // We also need to set up the TCCL to a ClassLoader that can access
        // the JAXB2 implementation packages
        ClassLoader old = Thread.currentThread().getContextClassLoader();
        try {
            Thread.currentThread().setContextClassLoader(Bus.class.getClassLoader());

            // Init
            jaxWsServiceFactoryBean.create();
        } finally {
            Thread.currentThread().setContextClassLoader(old);
        }

        // Build the ServerFactory
        JaxWsServerFactoryBean factoryBean = new EasyBeansJaxWsServerFactoryBean(jaxWsServiceFactoryBean,
                                                                                 factory);

        // Build the EasyBeans endpoint
        EasyBeansEndpoint endpoint = new EasyBeansEndpoint(factory, this.bus, factoryBean);
        endpoint.setServiceFactory(jaxWsServiceFactoryBean);

        // Set some endpoint properties
        Map<String, Object> properties = new HashMap<String, Object>();
        if (jaxWsImplementorInfo.getServiceName() != null) {
            properties.put(Endpoint.WSDL_SERVICE, jaxWsImplementorInfo.getServiceName());
        }
        if (jaxWsImplementorInfo.getEndpointName() != null) {
            properties.put(Endpoint.WSDL_PORT, jaxWsImplementorInfo.getEndpointName());
        }
        endpoint.setProperties(properties);
        return endpoint;
    }

    /**
     * Called when container is stopping.
     * @param info some information on the container which is stopping.
    */
    @Override
    public void stop(final EZBContainerCallbackInfo info) {

        stop();
        logger.info("Bus stopped for " + info.getArchive());

    }

    /**
     * This stop method is not directly called by the EJB container. The lifecycle object
     * itself has decided to stop (maybe one of its dependencies is now unavailable)
     */
    public void stop() {

        List<WSException> exceptions = new ArrayList<WSException>();

        for (IWebServiceEndpoint endpoint : endpoints) {
            try {
                // Stop and unregister
                endpoint.stop();
                manager.unregisterWSEndpoint(endpoint);

                // TODO unpublish WSDL ?
            } catch (WSException e) {
                // Do not rethrow the Exception directly: try to
                // undeploy the other endpoints before
                exceptions.add(e);
            }
        }

        // Clear the list
        endpoints.clear();

        // Stop the Bus
        if (this.bus != null) {
            // Bus start/stop operations are extra verbose
            Level old = setComponentLogLevel("org.apache.cxf", BasicLevel.LEVEL_ERROR);
            Level old2 = setComponentLogLevel("org.springframework", BasicLevel.LEVEL_ERROR);

            this.bus.shutdown(true);
            this.bus = null;

            // Reset log levels
            setComponentLogLevel("org.apache.cxf", old);
            setComponentLogLevel("org.springframework", old2);
        }

        // Warn the user for each of the unregistration that have failed
        // TODO add more details (names, ...)
        for (WSException exception : exceptions) {
            logger.warn("Unable to stop/unregister endpoint", exception);
        }
    }

    /**
     * Set the WSDLPublisherManager service.
     * @param publisherManager publication tool
     */
    public void setWSDLPublisherManager(WSDLPublisherManager publisherManager) {
        this.publisherManager = publisherManager;
    }
}
