/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
*/

package org.ow2.jonas.ws.publish.internal.manager;

import org.ow2.jonas.lib.service.AbsServiceImpl;
import org.ow2.jonas.service.ServiceException;
import org.ow2.jonas.ws.WSServiceException;
import org.ow2.jonas.ws.publish.WSDLPublisher;
import org.ow2.jonas.ws.publish.PublicationInfo;
import org.ow2.jonas.ws.publish.WSDLPublisherException;
import org.ow2.jonas.ws.publish.WSDLPublisherManager;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import javax.wsdl.Definition;
import java.util.ArrayList;
import java.util.List;


/**
 * Delegate publishing to all added WSDL Publishers.
 * @author Xavier Delplanque
 * @author Guillaume Sauthier
 */
public class DefaultWSDLPublisherManager extends AbsServiceImpl implements WSDLPublisherManager {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(DefaultWSDLPublisherManager.class);

    /**
     * List of WSDL Publishers in charge of WSDL Publication
     */
    private List<WSDLPublisher> publishers;

    /**
     * Get a new instance of WSDLManager.
     */
    public DefaultWSDLPublisherManager() {
        publishers = new ArrayList<WSDLPublisher>();
    }

    /**
     * @param param List of WSDL Publishers to create.
     */
    public void setPublishers(final String param) {
        List<String> handlers = convertToList(param);

        WSDLPublisherFactory factory = WSDLPublisherFactory.newInstance();
        for(String name : handlers) {
            // Create the publisher
            WSDLPublisher publisher = null;
            try {
                publisher = factory.newPublisher(name);
            } catch (WSDLPublisherException e) {
                logger.info("Cannot add the new WSDLPublisher '{0}'", name, e);
            }

            logger.debug("Adding WSDLPublisher ''{0}''", name);

            // Add the created Handler
            this.addPublisher(publisher);
        }
    }

    /**
     * Add a new publisher in this manager.
     * @param publisher new WSDLPublisher
     */
    public void addPublisher(final WSDLPublisher publisher) {
        publishers.add(publisher);
    }

    /**
     * Remove an already registered publisher.
     * @param publisher WSDLPublisher to be removed
     */
    public void removePublisher(WSDLPublisher publisher) {
        publishers.remove(publisher);
    }

    /**
     * Delegate publishing to all added WSDL Publishers.
     *
     * @param definition the WSDL file to be published.
     * @param info Publication info to help publication process
     *
     * @throws WSServiceException When publication fails.
     */
    public void publish(Definition definition, PublicationInfo info) throws WSDLPublisherException {
        for (WSDLPublisher publisher : publishers) {
            publisher.publish(definition, info);
        }
    }

    /**
     * Abstract start-up method to be implemented by sub-classes.
     *
     * @throws org.ow2.jonas.service.ServiceException
     *          service start-up failed
     */
    protected void doStart() throws ServiceException {
        // There is nothing to start in this service
    }

    /**
     * Abstract method for service stopping to be implemented by sub-classes.
     *
     * @throws org.ow2.jonas.service.ServiceException
     *          service stopping failed
     */
    protected void doStop() throws ServiceException {
        // There is nothing to stop in this service
    }
}
